//
//  Gradient.h
//  GradientDemo
//
//  Created by Guanshan Liu on 31/07/2012.
//  Copyright (c) 2012 Guanshan Liu. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <QuartzCore/QuartzCore.h>

typedef NS_ENUM(NSUInteger, GradientDrawingOptions) {
    GradientDrawsBeforeStartingLocation = (1 << 0),
    GradientDrawsAfterEndingLocation    = (1 << 1),
};

@interface Gradient : NSObject <NSCopying, NSCoding> {
    @private
    CFMutableArrayRef _colors;
    CGFloat *_locations;
    CGColorSpaceRef _colorSpace;
}

// Initialization
- (id)initWithStartingColor:(UIColor *)startingColor endingColor:(UIColor *)endingColor;
- (id)initWithColors:(NSArray *)colorArray;
- (id)initWithColorsAndLocations:(UIColor *)firstColor, ... NS_REQUIRES_NIL_TERMINATION;
- (id)initWithColors:(NSArray *)colorArray atLocations:(const CGFloat *)locations colorSpace:(CGColorSpaceRef)colorSpace;

// Primitive Drawing Methods
- (void)drawFromPoint:(CGPoint)startingPoint toPoint:(CGPoint)endingPoint options:(GradientDrawingOptions)options;
- (void)drawFromCenter:(CGPoint)startCenter radius:(CGFloat)startRadius toCenter:(CGPoint)endCenter radius:(CGFloat)endRadius options:(GradientDrawingOptions)options;

// Drawing Linear Gradients
- (void)drawInRect:(CGRect)rect angle:(CGFloat)angle;
- (void)drawInBezierPath:(UIBezierPath *)path angle:(CGFloat)angle;

// Drawing Radial Gradients
- (void)drawInRect:(CGRect)rect relativeCenterPosition:(CGPoint)relativeCenterPosition;
- (void)drawInBezierPath:(UIBezierPath *)path relativeCenterPosition:(CGPoint)relativeCenterPosition;

// Getting Gradient Properties
- (CGColorSpaceRef)colorSpace;
- (NSInteger)numberOfColorStops;
- (void)getColor:(UIColor **)color location:(CGFloat *)location atIndex:(NSInteger)index;
- (UIColor *)interpolatedColorAtLocation:(CGFloat)location;

@end
